unit VESAUnit;

interface

uses Dos;

type NullString = Array [0..255] of Char;

type ModeList = Array [0..32766] of Word;

type VBE_Info = record
  VESASignature : LongInt; {'VESA'}
  Version       : Word;
  OEMStringPTR  : ^NullString;
  Capabilities  : LongInt;
  VideoModePTR  : ^ModeList;
  TotalMemory   : Word;
  Reserved      : Array [21..256] of Byte;
end;

type Mode_Info = record
  ModeAttributes     : Word;
  WinAAttributes     : Byte;
  WinBAttributes     : Byte;
  WinGranularity     : Word;
  WinSize            : Word;
  WinASegment        : Word;
  WinBSegment        : Word;
  WinFuncPtr         : Pointer;
  BytesPerScanLine   : Word;
  XResolution        : Word;
  YResolution        : Word;
  XCharSize          : Byte;
  YCharSize          : Byte;
  NumberOfPlanes     : Byte;
  BitsPerPixel       : Byte;
  NumberOfBanks      : Byte;
  MemoryModel        : Byte;
  BankSize           : Byte;
  NumberOfImagePages : Byte;
  Reserved           : Array [31..256] of Byte;
end;

var
  VBEInfo          : ^VBE_Info;
  ModeInfo         : ^Mode_Info;
  Regs             : Registers;
  WinASegment,
  WinBSegment,
  ReadSegment,
  ReadType,
  BytesPerScanLine : Word;
  CurBank,
  CurReadBank      : Byte;
  BankAddr         : Array [0..15] of Word;

function IsVBE : Boolean;
function OEMString : String;
function SupportVideoMode (Mode : Word): Boolean;
procedure VESAGetModeInfo (Mode : Word);
function VESAFindMode (XRes, YRes : LongInt; BitsPerPixel : Byte) : Word;
function VESASetMode (Mode : Word) : Boolean;
function VESASwitchBank (Bank : Word) : Boolean;
function VESASwitchReadBank (Bank : Word) : Boolean;
procedure VESAPutPixel (X, Y : LongInt; Color : Byte);
function VESAGetPixel (X, Y : LongInt) : Byte;

implementation

{This function checks whether a VESA BIOS is installed}
function IsVBE : Boolean;
begin
  {Assign memory if not yet assigned}
  if VBEInfo = nil then
    New (VBEInfo);

  {Function 0 - Initialize}
  Regs.AX := $4F00;
  Regs.ES := Seg (VBEInfo^);
  Regs.DI := Ofs (VBEInfo^);
  Intr ($10, Regs);

  {Return success if AX = 4F}
  IsVBE := Regs.AX = $4F;
end;

{This function returns the "Original Equipment Manufacturer" ID String.
This tells you who made the graphics card. The string is converted from
a null-terminated string to a standard pascal string}
function OEMString : String;
var
  Temp : String;
  Counter : Byte;

begin
  Counter := 0;
  Temp := '';

  repeat
    Temp := Temp + VBEInfo^.OEMStringPTR^ [Counter];
    inc (Counter);
  until VBEInfo^.OEMStringPTR^ [Counter] = Chr (0);

  OEMString := Temp;
end;

{This functions checks whether a specific video mode is supported}
function SupportVideoMode (Mode : Word): Boolean;
var
  Counter : Word;

begin
  Counter := 0;
  repeat
    {If the video mode is found in the list, return success}
    if VBEInfo^.VideoModePTR^ [Counter] = Mode then begin
      SupportVideoMode := true;
      Exit;
    end;
    {Move on to the next entry in the list}
    inc (Counter);
    {Until the end is reached}
  until VBEInfo^.VideoModePTR^ [Counter] = $FFFF;
  {Return failure}
  SupportVideoMode := false;
end;

procedure VESAGetModeInfo (Mode : Word);
begin
  {Assign a memory if not yet assigned}
  if ModeInfo = nil then
    New (ModeInfo);

  {Function 1 - Get graphics mode info}
  Regs.AX := $4F01;
  Regs.CX := Mode;
  Regs.ES := Seg (ModeInfo^);
  Regs.DI := Ofs (ModeInfo^);
  Intr ($10, Regs);
end;

function VESAFindMode (XRes, YRes : LongInt; BitsPerPixel : Byte) : Word;
var
  TestMode, Counter : Word;

begin
  Counter := 0;

  repeat
    TestMode := VBEInfo^.VideoModePTR^ [Counter];
    {At the end of the list, return failure}
    if (TestMode = $FFFF) then begin
      VESAFindMode := 0;
      Exit;
    end;
    {Check whether this is correct mode, if yes, return it}
    VESAGetModeInfo (TestMode);
    if (ModeInfo^.XResolution = XRes) and
       (ModeInfo^.YResolution = YRes) and
       (ModeInfo^.BitsPerPixel = BitsPerPixel) then begin
      VESAFindMode := TestMode;
      Exit;
    end;
    {Move on to the next entry in the list}
    inc (Counter);
  until false;
end;

function VESASetMode (Mode : Word) : Boolean;
var
  Counter : Word;

begin
  {Function 2 - Set graphics mode}
  Regs.AX := $4F02;
  Regs.BX := Mode;
  Intr ($10, Regs);

  {Return success if AX = 4F}
  VESASetMode := Regs.AX = $4F;

  {Get information on the graphics mode}
  VESAGetModeInfo (Mode);

  {Get the addresses of the banks in linear display memory and store them}
  if (ModeInfo^.WinGranularity > 0) then
  for Counter := 0 to 15 do begin
    BankAddr [Counter] := Counter * 64 div ModeInfo^.WinGranularity;
  end;

  WinASegment      := ModeInfo^.WinASegment;
  WinBSegment      := ModeInfo^.WinBSegment;
  BytesPerScanLine := ModeInfo^.BytesPerScanLine;

  if (ModeInfo^.WinAAttributes and 2 = 2) then begin
    ReadSegment := WinASegment;
    ReadType := 0;
  end else begin
    ReadSegment := WinBSegment;
    ReadType := 1;
  end;
end;

function VESASwitchBank (Bank : Word) : Boolean;
begin
  CurBank := Bank;

  {Function 5 - Set SuperVGA window}
  Regs.AX := $4F05;
  Regs.BX := 0;
  Regs.DX := BankAddr [Bank];
  Intr ($10, Regs);

  {Return success if AX = 4F}
  VESASwitchBank := Regs.AX = $4F;
end;

function VESASwitchReadBank (Bank : Word) : Boolean;
begin
  CurReadBank := Bank;

  {Function 5 - Set SuperVGA window}
  Regs.AX := $4F05;
  Regs.BX := ReadType;
  Regs.DX := BankAddr [Bank];
  Intr ($10, Regs);

  {Return success if AX = 4F}
  VESASwitchReadBank := Regs.AX = $4F;
end;

procedure VESAPutPixel (X, Y : LongInt; Color : Byte);
var
  OffSet : LongInt;
  Bank, Pixel : Word;

begin
  {Calculate linear offset in display memory}
  OffSet := BytesPerScanLine * Y + X;
  {Calculate offset in 64K segment}
  Pixel := OffSet;
  {Calculate bank}
  Bank := longint (OffSet shr 16);

  {Switch banks if necessary}
  if CurBank <> Bank then
    VESASwitchBank (Bank);

  {Set pixel}
  Mem [WinASegment:Pixel] := Color;
end;

function VESAGetPixel (X, Y : LongInt) : Byte;
var
  OffSet : LongInt;
  Bank, Pixel : Word;

begin
  {Calculate linear offset in display memory}
  OffSet := BytesPerScanLine * Y + X;
  {Calculate offset in 64K segment}
  Pixel := OffSet;
  {Calculate bank}
  Bank := longint (OffSet shr 16);

  {Switch banks if necessary}
  if CurReadBank <> Bank then
    VESASwitchReadBank (Bank);

  {Set pixel}
  VESAGetPixel := Mem [ReadSegment:Pixel];
end;

end.